<?php

namespace Drupal\testsuite\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\testsuite\LoadResourceService;
use Drupal\testsuite\TestMenuService;

/**
 * Configure example settings for this site.
 */
class CustomTestsForm extends ConfigFormBase {

  /** 
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'testsuite.custom';

  /**
   * @var \Drupal\testsuite\LoadResourceService
   */
  protected $_loadResourceService;

  /**
   * @var \Drupal\testsuite\TestMenuService
   */
  protected $_testMenuService;

  /**
   * @var array The array of files to create Resource with.
   */
  protected $_loadedFiles = [];

  /**
   * @param \Drupal\testsuite\LoadResourceService $loadResourceService
   * @param \Drupal\testsuite\TestMenuService $testMenuService
   */
  public function __construct(LoadResourceService $loadResourceService, TestMenuService $testMenuService) {
    $this->_testMenuService = $testMenuService;
    $this->_loadResourceService = $loadResourceService;
    foreach($this->_loadResourceService->getResource('form', 'custom', 'testsuite', 'all', 'all') as $file) {
      $className = basename($file, '.php');
      $class = '\Drupal\testsuite\Tests\\'.$className;
      require_once $file;
      if(class_exists($class)) {
        $obj = new $class;
        if (\Drupal::moduleHandler()->moduleExists($obj->getModuleName()) || $obj->getModuleName() == '') {
          $this->_loadedFiles[$className] = array(
            'file' => $file,
            'moduleName' => $obj->getModuleName(),
            'name' => $obj->getName(),
            'description' => $obj->getDescription(),
            'class' => '\Drupal\testsuite\Tests\\'.$className,
          );
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    return new static(
      // Load the service required to construct this class.
      $container->get('testsuite.load_resource.service'),
      $container->get('testsuite.test_menu.service')
    );
  }

  /** 
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'testsuite_custom_tests_form';
  }

  /** 
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /** 
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['menu'] = $this->_testMenuService->buildTestsMenu();

    foreach($this->_loadedFiles as $className => $values) {
      // var_dump($values['moduleName']);
      // var_dump(\Drupal::moduleHandler()->moduleExists($values['moduleName']));
      $form[$className] = [
        '#type' => 'checkbox',
        '#title' => $this->t($values['name']),
        '#attributes' => array(
          'id' => $className,
          'name' => $className,
        ),
        '#default_value' => TRUE,
      ];
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#attributes' => array(
        'class' => ['button', 'button-primary']
      ),
      '#value' => $this->t('Submit')
    ];

    return $form;
  }

  /** 
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // \Drupal::messenger()->addStatus('Form Submitted.');
    foreach($this->_loadedFiles as $className => $values) {
      // \Drupal::messenger()->addStatus($className . ": " . $form_state->getValue($className));
      if($form_state->hasValue($className) && $form_state->getValue($className) == TRUE) {
        require_once $values['file'];
        if(class_exists($values['class'])) {
          $obj = new $values['class'];
          $request = $this->getRequest();
          $results = $obj->runTest($request);
          $request->getSession()->set($className, $results);
          // \Drupal::messenger()->addStatus($obj->getName() . ": " . $results);
          $form_state->setRedirect('testsuite.custom_report');
        }
      }
    }
  }

}