<?php

namespace Drupal\testsuite;

use Drupal\testsuite\LoadResourceService;
use Drupal\Core\Link;
use \Drupal\Core\Url;
use Drupal\Core\Render\Markup;

/**
 * Test Suite Services.
 */
class TestMenuService {

    /**
     * @var \Drupal\testsuite\LoadResourceService
     */
    protected $_loadResourceService;

    /**
     *@param \Drupal\testsuite\LoadResourceService $loadResourceService
     */
    public function __construct(LoadResourceService $loadResourceService) {
        $this->_loadResourceService = $loadResourceService;
    }
    
    /**
     * Gets the array of files to create classes with.
     *
     * @return array
     *   The array of files to create classes with.
     */
    public function buildTestsMenu() {
        $customUrl = Url::fromRoute('testsuite.custom');
        // \Drupal::logger('testsuite')->info("here.");
        $phpunitUrl = Url::fromRoute('testsuite.phpunit', ['area' => 'custom', 'module' => 'testsuite']);
        // \Drupal::logger('testsuite')->info("here2.");
        $seleniumUrl = Url::fromRoute('testsuite.selenium');
        $custom_link = Link::fromTextAndUrl(t('Custom'), $customUrl);
        $phpunit_link = Link::fromTextAndUrl(t('PHPUnit'), $phpunitUrl);
        $selenium_link = Link::fromTextAndUrl(t('Selenium'), $seleniumUrl);
        // $project_link = $project_link->toRenderable();
        $list_items = array(
            $phpunit_link,
            $custom_link,
            $selenium_link
        );

        $output['page_menu'] = [
            '#theme' => 'item_list',
            '#type' => 'ul',
            '#items' => $list_items,
            '#attached' => [
                'library' => [
                    'testsuite/menu'
                ]
            ],
            '#attributes' => array('class' => 'test_suite__list'),
            '#prefix' => '<div class="test-suite-page-menu-container">',
            '#suffix' => '</div>',
            
        ];

        return $output;
    }

    private function _createName($machine_name) {
        $parts = explode("_", $machine_name);
        return ucwords(implode(" ", $parts));
    }

    /**
     * Gets the array of files to create classes with.
     *
     * @return array
     *   The array of files to create classes with.
     */
    public function buildModuleMenu() {
        $modulesArray = $this->_loadResourceService->getResource('menu', 'phpunit', 'all', 'all', 'all');
        $list_items = array();
        $all_items = array();
        $linkString = '';
        // \Drupal::logger('testsuite')->info(json_encode($modulesArray['contrib']));
        $javaScriptUrl = Url::fromUserInput('#', ['fragment' => '#']);
        foreach($modulesArray as $area => $modules) {
            $items = array();
            foreach($modules as $module => $testDirectory) {
                $directories = array();
                
                foreach($testDirectory as $directory) {
                    if(!in_array($directory, $all_items)) {
                        $all_items[] = $directory;
                    }
                    $link = Link::fromTextAndUrl(
                        t($directory),
                        Url::fromRoute(
                            'testsuite.phpunit',
                            [
                                'area' => $area,
                                'module' => $module,
                                'directory' => $directory
                            ]
                        )
                    )->toRenderable();
                    $link['#attributes'] = [
                        'class' => ['test-suite-directory-links']
                    ];
                    $directories[] = $link;
                }
                $items[] = [
                    '#markup' => '<a href="#" class="test-suite-module-links">'.t($this->_createName($module)).'<i class="arrow down"></i></a>',
                    'children' => $directories
                ];
            }
            
            /* foreach($all_items as $directory) {
                $linkString .= '<a href="/admin/testsuite/phpunit/'.$area.'/all/'.$directory.'" class="button button--notice test-suite-all-directory-links">'.t(ucwords('all '.$area.' '.$directory)).'</a>';
            } */
            $list_items[] = [
                '#markup' => '<a href="#" class="test-suite-area-links">'.ucwords($area).'</a>',
                'children' => $items
            ];
        }


        $output['modules_menu'] = [
            '#theme' => 'item_list',
            '#type' => 'ul',
            '#attributes' => array('class' => 'test_suite__list'),
            '#items' => $list_items,
            '#prefix' => '<div class="test-suite-module-menu-container">',
            '#suffix' => '</div>',
        ];

        /* $output['all_menu'] = [
            '#type' => 'markup',
            '#markup' => $linkString,
            '#prefix' => '<div class="test-suite-all-menu-container">',
            '#suffix' => '</div>',
        ]; */

        return $output;
    }
}